@@ -10,16 +10,21 @@
 # This module handles dsconf libdb sub-commands
 #
 
+import sys
 import os
-import re
 import glob
+import pwd
+import re
+import logging
 import shutil
-from lib389.dseldif import DSEldif
-from lib389._constants import DEFAULT_LMDB_SIZE
-from lib389.cli_base import CustomHelpFormatter
-from lib389.utils import parse_size, format_size
 import subprocess
+from enum import Enum
 from errno import ENOSPC
+from lib389.cli_base import CustomHelpFormatter
+from lib389._constants import DEFAULT_LMDB_SIZE, BDB_IMPL_STATUS, DN_CONFIG, DBSCAN
+from lib389.dseldif import DSEldif
+from lib389.utils import parse_size, format_size, check_plugin_strings, find_plugin_path
+from pathlib import Path
 
 
 DBLIB_LDIF_PREFIX = "__dblib-"
@@ -31,6 +36,7 @@ MDB_MAP = "data.mdb"
 MDB_LOCK = "lock.mdb"
 
 LDBM_DN = "cn=config,cn=ldbm database,cn=plugins,cn=config"
+CL5DB='replication_changelog.db'
 
 _log = None
 
@@ -44,6 +50,120 @@ class FakeArgs(dict):
         self.__dict__ = self
 
 
+class CalledProcessUnexpectedReturnCode(subprocess.CalledProcessError):
+    def __init__(self, result, expected_rc):
+        super().__init__(cmd=result.args, returncode=result.returncode, output=result.stdout, stderr=result.stderr)
+        self.expected_rc = expected_rc
+        self.result = result
+
+    def __str__(self):
+        return f'Command {self.result.args} returned {self.result.returncode} instead of {self.expected_rc}'
+
+
+class DbscanHelper:
+    """
+    Helper class to get the existing database(s) dbis
+    Usage: DbscanHelper(inst).get_dbi('attrname' [, backend='backendname'])
+           throws KeyError if there is no associated dbi
+    """
+
+    def __init__(self, inst, log=None):
+        self.inst = inst
+        self.log = log if log else logging.getLogger(__name__)
+
+    def _init2(self):
+        if getattr(self, 'dblib', None) is not None:
+            return
+        assert(self.inst)
+        self.dblib = self.inst.get_db_lib()
+        self.dbhome = self.inst.ds_paths.db_dir
+        self.dbis = self._list_instances()
+        self.ldif_dir = self.inst.ds_paths.ldif_dir
+
+        # set self.options with dbscan supported options
+        self.options = []
+        usage = self.dbscan(['-h'], expected_rc=None).stdout
+        pattern = r'^\s+(?:(-[^-,]+), +)?(--[^ ]+).*$'
+        for match in re.finditer(pattern, usage, flags=re.MULTILINE):
+            for idx in range(1,3):
+                if match.group(idx) is not None:
+                    self.options.append(match.group(idx))
+
+    def resync(self):
+        assert(self.inst)
+        self.dblib = None
+        self._init2()
+
+    def _list_instances(self):
+        # compute db instance pathnames
+        instances = self.dbscan(['-D', self.dblib, '-L', self.dbhome]).stdout
+        dbis = []
+        if self.dblib == 'bdb':
+            pattern = r'^ (.*) $'
+        else:
+            pattern = r'^ (.*) flags:'
+        for match in re.finditer(pattern, instances, flags=re.MULTILINE):
+            dbis.append(match.group(1))
+        return dbis
+
+    def get_dbi(self, attr, backend='userroot'):
+        self._init2()
+        for dbi in self.dbis:
+            if f'{backend}/{attr}.'.lower() in dbi.lower():
+                return dbi
+        raise KeyError(f'Unknown dbi {backend}/{attr}')
+
+    def dbscan(self, args, expected_rc=0):
+        if self.inst is None:
+            prefix = os.environ.get('PREFIX', "")
+            prog = f'{prefix}/bin/dbscan'
+        else:
+            prog = os.path.join(self.inst.ds_paths.bin_dir, DBSCAN)
+        args.insert(0, prog)
+        output = subprocess.run(args, encoding='utf-8', stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
+        self.log.debug(f'{args} result is {output.returncode} output is {output.stdout}')
+        if expected_rc is not None and expected_rc != output.returncode:
+            raise CalledProcessUnexpectedReturnCode(output, expected_rc)
+        return output
+
+    def __repr__(self):
+        if self.inst is None:
+            return 'DbscanHelper(None)'
+        self._init2()
+        attrs = ['inst', 'dblib', 'dbhome', 'ldif_dir', 'options', 'dbis' ]
+        res = ", ".join(map(lambda x: f'{x}={getattr(self, x, None)}', attrs))
+        return f'DbscanHelper({res})'
+
+
+def get_bdb_impl_status():
+    backldbm = 'libback-ldbm'
+    bundledbdb_plugin = 'libback-bdb'
+    robdb_symbol = 'bdbro_getcb_vector'
+    libdb = 'libdb-'
+    plgstrs = check_plugin_strings(backldbm, [bundledbdb_plugin, robdb_symbol, libdb])
+    if plgstrs[bundledbdb_plugin] is True:
+        # bundled bdb build
+        if find_plugin_path(bundledbdb_plugin):
+            return BDB_IMPL_STATUS.BUNDLED
+        return BDB_IMPL_STATUS.NONE
+    if plgstrs[robdb_symbol] is True:
+        # read-only bdb build
+        return BDB_IMPL_STATUS.READ_ONLY
+    if plgstrs[libdb] is True:
+        # standard bdb package build
+        return BDB_IMPL_STATUS.STANDARD
+    # Unable to find libback-ldbm plugin
+    return BDB_IMPL_STATUS.UNKNOWN
+
+
+def is_bdb_supported(read_write=True):
+    bdbok = [BDB_IMPL_STATUS.BUNDLED, BDB_IMPL_STATUS.STANDARD]
+    if not read_write:
+        # READ_MODE is ok too
+        bdbok.append(BDB_IMPL_STATUS.READ_ONLY)
+    return get_bdb_impl_status() in bdbok
+
+
 def get_ldif_dir(instance):
     """
     Get the server's LDIF directory.
@@ -65,6 +185,7 @@ def get_backends(log, dse, tmpdir):
     # Lets be sure to keep config backend info
     ic2ec['config'] = 'config'
     update_dse = []
+    dbis = None
     for entry in dse._contents:
         found = re.search(r'^nsslapd-backend: (.*)', entry)
         if found:
@@ -74,22 +195,28 @@ def get_backends(log, dse, tmpdir):
             dn = found[0][4:]
             bename = found[1].lower()
             if not bename in ic2ec:
-                # Not a mapping tree backend 
+                # Not a mapping tree backend
                 continue
             ecbename = ic2ec[bename]
             suffix = dse.get(dn, "nsslapd-suffix", True)
             dbdir = dse.get(dn, "nsslapd-directory", True)
             ecdbdir = dbdir
-            if dbdir is None and 'config' in res:
+            dblib = dse.get(dn, "nsslapd-backend-implement", True)
+            if dblib is None and 'config' in res:
+                dblib = res['config']['dblib']
+            if dbis is None:
+                dbis = get_mdb_dbis(dbdir) if dblib == 'mdb' else []
+            # in bdb case dbdir should be the database instance directory
+            # in mdb case it is the database map file directory
+            if 'config' in res and ( dbdir is None or dblib == 'mdb'):
                 dbdir = f"{res['config']['dbdir']}/{bename}"
                 ecdbdir = f"{res['config']['dbdir']}/{ecbename}"
                 # bdb requires nsslapd-directory so lets add it once reading is done.
                 update_dse.append((dn, ecdbdir))
-            dblib = dse.get(dn, "nsslapd-backend-implement", True)
             ldifname = f'{tmpdir}/{DBLIB_LDIF_PREFIX}{bename}.ldif'
             cl5name = f'{tmpdir}/{DBLIB_LDIF_PREFIX}{bename}.cl5.dbtxt'
-            cl5dbname = f'{dbdir}/replication_changelog.db'
-            eccl5dbname = f'{ecdbdir}/replication_changelog.db'
+            cl5dbname = f'{dbdir}/{CL5DB}'
+            eccl5dbname = f'{ecdbdir}/{CL5DB}'
             dbsize = 0
             entrysize = 0
             for f in glob.glob(f'{dbdir}/id2entry.db*'):
@@ -99,8 +226,14 @@ def get_backends(log, dse, tmpdir):
             indexes = dse.get_indexes(bename)
             # Let estimate the number of dbis: id2entry + 1 per regular index + 2 per vlv index
             dbi = 1 + len(indexes) + len([index for index in indexes if index.startswith("vlv#")])
+            if dblib == "bdb":
+                has_changelog = os.path.isfile(eccl5dbname)
+            elif bename in dbis:
+                has_changelog = CL5DB in dbis[bename]
+            else:
+                has_changelog = False
 
-            res[bename] = ({
+            res[bename] = {
                 'dn': dn,
                 'bename': bename,
                 'ecbename': ecbename,
@@ -116,13 +249,15 @@ def get_backends(log, dse, tmpdir):
                 'dbsize': dbsize,
                 'entrysize': entrysize,
                 'indexes': indexes,
+                'has_changelog': has_changelog,
                 'dbi': dbi
-            })
+            }
+
     # now that we finish reading the dse.ldif we may update it if needed.
     for dn, dir in update_dse:
         dse.replace(dn, 'nsslapd-directory', dir)
     log.debug(f'lib389.cli_ctl.dblib.get_backends returns: {str(res)}')
-    return res
+    return (res, dbis)
 
 
 def get_mdb_dbis(dbdir):
@@ -160,8 +295,11 @@ def run_dbscan(args):
 
 def export_changelog(be, dblib):
     # Export backend changelog
+    if not be['has_changelog']:
+        return False
     try:
         cl5dbname = be['eccl5dbname'] if dblib == "bdb" else be['cl5dbname']
+        _log.info(f"Exporting changelog {cl5dbname} to {be['cl5name']}")
         run_dbscan(['-D', dblib, '-f', cl5dbname, '-X', be['cl5name']])
         return True
     except subprocess.CalledProcessError as e:
@@ -172,7 +310,8 @@ def import_changelog(be, dblib):
     # import backend changelog
     try:
         cl5dbname = be['eccl5dbname'] if dblib == "bdb" else be['cl5dbname']
-        run_dbscan(['-D', dblib, '-f', cl5dbname, '-I', be['cl5name']])
+        _log.info(f"Importing changelog {cl5dbname} from {be['cl5name']}")
+        run_dbscan(['-D', dblib, '-f', cl5dbname, '--import', be['cl5name'], '--do-it'])
         return True
     except subprocess.CalledProcessError as e:
         return False
@@ -189,6 +328,15 @@ def set_owner(list, uid, gid):
             pass
 
 
+def get_uid_gid_from_dse(dse):
+    # For some reason inst.get_user_uid() always returns dirsrv uid
+    # even for non root install.
+    # Lets looks directly in the dse.ldif intead
+    user = dse.get(DN_CONFIG, 'nsslapd-localuser', single=True)
+    pwent = pwd.getpwnam(user)
+    return ( pwent.pw_uid, pwent.pw_gid )
+
+
 def dblib_bdb2mdb(inst, log, args):
     global _log
     _log = log
@@ -202,12 +350,20 @@ def dblib_bdb2mdb(inst, log, args):
         log.error(f"Failed trying to create the directory {tmpdir} needed to store the ldif files, error: {str(e)}")
         return
 
+    status = get_bdb_impl_status()
+    if status is BDB_IMPL_STATUS.UNKNOWN:
+        log.warning('Unable to determine if Berkeley Database library is available. If it is not the case, the migration will fail.')
+    elif status is BDB_IMPL_STATUS.NONE:
+        log.error('Berkeley Database library is not available. Maybe 389-ds-base-bdb rpm should be installed.')
+        raise RuntimeError('Berkeley Database library is not available')
+
     # Cannot use Backends(inst).list() because it requires a connection.
     # lets use directlt the dse.ldif after having stopped the instance
 
     inst.stop()
     dse = DSEldif(inst)
-    backends = get_backends(log, dse, tmpdir)
+    uid, gid = get_uid_gid_from_dse(dse)
+    backends,dbis = get_backends(log, dse, tmpdir)
     dbmapdir = backends['config']['dbdir']
     dblib = backends['config']['dblib']
 
@@ -218,6 +374,8 @@ def dblib_bdb2mdb(inst, log, args):
     # Remove ldif files and mdb files
     dblib_cleanup(inst, log, args)
 
+    # Check if changelog is present in backends
+
     # Compute the needed space and the lmdb map configuration
     total_dbsize = 0
     total_entrysize = 0
@@ -247,8 +405,6 @@ def dblib_bdb2mdb(inst, log, args):
     log.info(f"Required number of dbi is {nbdbis}")
 
     # Generate the info file (so dbscan could generate the map)
-    uid = inst.get_user_uid()
-    gid = inst.get_group_gid()
     with open(f'{dbmapdir}/{MDB_INFO}', 'w') as f:
         f.write('LIBVERSION=9025\n')
         f.write('DATAVERSION=0\n')
@@ -335,7 +491,7 @@ def dblib_bdb2mdb(inst, log, args):
     dbhome=backends["config"]["dbdir"]
     set_owner(glob.glob(f'{dbhome}/*.mdb'), uid, gid)
     log.info("Backends importation 100%")
-    inst.start()
+    inst.start(post_open=False)
     log.info("Migration from Berkeley database to lmdb is done.")
 
 
@@ -352,16 +508,26 @@ def dblib_mdb2bdb(inst, log, args):
         log.error(f"Failed trying to create the directory {tmpdir} needed to store the ldif files, error: {str(e)}")
         return
 
+    status = get_bdb_impl_status()
+    if status is BDB_IMPL_STATUS.UNKNOWN:
+        log.warning('Unable to determine if Berkeley Database library is available. If it is not the case, the migration will fail.')
+    elif status is BDB_IMPL_STATUS.READ_ONLY:
+        log.error('It is not possible to migrate to read-only Berkeley Database library.')
+        raise RuntimeError('Berkeley Database library is read-only')
+    elif status is BDB_IMPL_STATUS.NONE:
+        log.error('Berkeley Database library is not available. Maybe 389-ds-base-bdb rpm should be installed.')
+        raise RuntimeError('Berkeley Database library is not available')
+
     # Cannot use Backends(inst).list() because it requires a connection.
     # lets use directlt the dse.ldif after having stopped the instance
 
     inst.stop()
     dse = DSEldif(inst)
-    backends = get_backends(log, dse, tmpdir)
+    uid, gid = get_uid_gid_from_dse(dse)
+    backends,dbis = get_backends(log, dse, tmpdir)
     dbmapdir = backends['config']['dbdir']
     dbhome = inst.ds_paths.db_home_dir
     dblib = backends['config']['dblib']
-    dbis = get_mdb_dbis(dbmapdir)
 
     if dblib == "bdb":
         log.error(f"Instance {inst.serverid} is already configured with bdb.")
@@ -410,8 +576,6 @@ def dblib_mdb2bdb(inst, log, args):
         if 'has_id2entry' not in be:
             continue
         total_dbsize += 1
-    uid = inst.get_user_uid()
-    gid = inst.get_group_gid()
     for bename, be in backends.items():
         # Keep only backend associated with a db
         if 'has_id2entry' not in be:
@@ -456,7 +620,7 @@ def dblib_mdb2bdb(inst, log, args):
     set_owner((f'{dbhome}/DBVERSION', f'{dbmapdir}/DBVERSION', f'{dbhome}/guardian', '{dbmapdir}/guardian'), uid, gid)
 
     log.info("Backends importation 100%")
-    inst.start()
+    inst.start(post_open=False)
     log.info("Migration from ldbm to Berkeley database is done.")
 
 
@@ -473,7 +637,7 @@ def dblib_cleanup(inst, log, args):
     _log = log
     tmpdir = get_ldif_dir(inst)
     dse = DSEldif(inst)
-    backends = get_backends(log, dse, tmpdir)
+    backends,dbis = get_backends(log, dse, tmpdir)
     dbmapdir = backends['config']['dbdir']
     dbhome = inst.ds_paths.db_home_dir
     dblib = backends['config']['dblib']
@@ -484,14 +648,19 @@ def dblib_cleanup(inst, log, args):
         # Keep only backend associated with a db
         if 'has_id2entry' not in be and be['dbsize'] == 0:
             continue
-        rm(be['ldifname'])
-        rm(be['cl5name'])
-        dbdir = be['dbdir']
-        if dblib == "mdb":
-            # remove db dir
-            for f in glob.glob(f'{dbdir}/*.db*'):
+        # rm(be['ldifname'])
+        # rm(be['cl5name'])
+
+    if dblib == "mdb":
+        # Looks for bdb database instance subdirectories
+        for id2entry in glob.glob(f'{dbmapdir}/*/id2entry.db*'):
+            dbdir = Path(id2entry).parent.absolute()
+            log.info(f"cleanup removing {dbdir}")
+            for f in glob.iglob(f'{dbdir}/*.db*'):
                 rm(f)
-            rm(f'{dbdir}/DBVERSION')
+            for f in [ 'guardian', 'DBVERSION' ]:
+                if os.path.isfile(f'{dbdir}/{f}'):
+                    rm(f'{dbdir}/{f}')
             os.rmdir(dbdir)
 
     if dblib == "bdb":
@@ -499,13 +668,13 @@ def dblib_cleanup(inst, log, args):
         rm(f'{dbmapdir}/data.mdb')
         rm(f'{dbmapdir}/lock.mdb')
     else:
-        for f in glob.glob(f'{dbhome}/__db.*'):
+        for f in glob.iglob(f'{dbhome}/__db.*'):
             rm(f)
-        for f in glob.glob(f'{dbmapdir}/__db.*'):
+        for f in glob.iglob(f'{dbmapdir}/__db.*'):
             rm(f)
-        for f in glob.glob(f'{dbhome}/log.*'):
+        for f in glob.iglob(f'{dbhome}/log.*'):
             rm(f)
-        for f in glob.glob(f'{dbmapdir}/log.*'):
+        for f in glob.iglob(f'{dbmapdir}/log.*'):
             rm(f)
         rm(f'{dbhome}/DBVERSION')
         rm(f'{dbmapdir}/DBVERSION')
