@@ -27,10 +27,13 @@ except ImportError:
 import json
 import re
 import os
+import glob
 import logging
 import shutil
 import ldap
+import mmap
 import socket
+import ipaddress
 import time
 import stat
 from datetime import (datetime, timedelta)
@@ -187,6 +190,10 @@ _chars = {
 SIZE_UNITS = { 't': 2**40, 'g': 2**30, 'm': 2**20, 'k': 2**10, '': 1, }
 SIZE_PATTERN = r'\s*(\d*\.?\d*)\s*([tgmk]?)b?\s*'
 
+RPM_TOOL = '/usr/bin/rpm'
+LDD_TOOL = '/usr/bin/ldd'
+OBJDUMP_TOOL = '/usr/bin/objdump'
+
 #
 # Utilities
 #
@@ -1707,6 +1714,45 @@ def is_valid_hostname(hostname):
     allowed = re.compile(r"(?!-)[A-Z\d-]{1,63}(?<!-)$", re.IGNORECASE)
     return all(allowed.match(x) for x in hostname.split("."))
 
+def is_valid_ip(ip):
+    """ Validate an IPv4 or IPv6 address, including asterisks for wildcards. """
+    if '*' in ip and '.' in ip:
+        ipv4_pattern = r'^(\d{1,3}|\*)\.(\d{1,3}|\*)\.(\d{1,3}|\*)\.(\d{1,3}|\*)$'
+        if re.match(ipv4_pattern, ip):
+            octets = ip.split('.')
+            for octet in octets:
+                if octet != '*':
+                    try:
+                        val = int(octet, 10)
+                        if not (0 <= val <= 255):
+                            return False
+                    except ValueError:
+                        return False
+            return True
+        else:
+            return False
+
+    if '*' in ip and ':' in ip:
+        ipv6_pattern = r'^([0-9a-fA-F]{1,4}|\*)(:([0-9a-fA-F]{1,4}|\*)){0,7}$'
+        if re.match(ipv6_pattern, ip):
+            octets = ip.split(':')
+            for octet in octets:
+                if octet != '*':
+                    try:
+                        val = int(octet, 16)
+                        if not (0 <= val <= 0xFFFF):
+                            return False
+                    except ValueError:
+                        return False
+            return True
+        else:
+            return False
+
+    try:
+        ipaddress.ip_address(ip)
+        return True
+    except ValueError:
+        return False
 
 def parse_size(size):
     """
@@ -1759,7 +1805,7 @@ def get_default_mdb_max_size(paths):
     # otherwise decrease the value
     dbdir = paths.db_dir
     # dbdir may not exists because:
-    #  - paths instance name has not been expanded 
+    #  - paths instance name has not been expanded
     #  - dscreate has not yet created it.
     # so let use the first existing parent in that case.
     while not os.path.exists(dbdir):
@@ -1769,7 +1815,7 @@ def get_default_mdb_max_size(paths):
         avail = statvfs.f_frsize * statvfs.f_bavail
         avail *= 0.8 # Reserve 20% as margin
         if size > avail:
-            mdb_max_size = str(avail)
+            mdb_max_size = format_size(avail)
     except (TimeoutError, InterruptedError) as e:
         raise e
     except OSError as e:
@@ -1984,3 +2030,44 @@ def get_passwd_from_file(passwd_file):
             passwd = f.readline().strip()
             return passwd
     raise ValueError(f"The password file '{passwd_file}' does not exist, or can not be read.")
+
+
+def find_plugin_path(plugin_name):
+    p = Paths()
+    plugin = None
+    for ppath in glob.glob(f'{p.plugin_dir}/{plugin_name}.*'):
+        if not ppath.endswith('.la'):
+            plugin = ppath
+    return plugin
+
+
+def check_plugin_strings(plugin_name, tested_strings):
+    """
+    Returns a dict mapping each tested symbol to True, False or None
+    """
+    plugin = find_plugin_path(plugin_name)
+    if plugin:
+        # Otherwise looks directly for the string in the plugin
+        with open(plugin, "rb") as f:
+            with mmap.mmap(f.fileno(), 0, access=mmap.ACCESS_READ) as mm:
+                return { astring:(mm.find(astring.encode()) >=0) for astring in tested_strings }
+    return { astring:None for astring in tested_strings }
+
+
+def get_timeout_scale():
+    """
+    Get the timeout scale factor
+    :return float:
+    """
+    scale_factor = 1.0
+    if Paths().asan_enabled:
+        scale_factor = 2.0
+
+    env_value = os.getenv('DS_TIMEOUT_SCALE', default=str(scale_factor))
+
+    try:
+        return float(env_value)
+    except ValueError:
+        log.error(f"DS_TIMEOUT_SCALE should be a valid float. Using default value: {scale_factor}")
+        return scale_factor
+
